#include <stdint.h>
#include "resources.h"

/*
 * Ozone CS pin
 ******************************************************************************/
sbit OZONE_CS at PORTA5_bit;
sbit OZONE_CS_DIR at DDA5_bit;

/*
 * Global vars
 ******************************************************************************/
const float    Rr    = 10000.0;
const float    Vref  = 5.0;
uint16_t       raw;
float          ozone;
char           text[ 20 ];

/*
 * Prototypes
 ******************************************************************************/
void system_init( void );
void display_result( void );
void measurement( void );
void calculation( void );

/*
 * Functions
 ******************************************************************************/
void system_init()
{
    OZONE_CS_DIR = 1;
    Delay_ms( 200 );

    SPI1_Init_Advanced( _SPI_MASTER, _SPI_FCY_DIV8, _SPI_CLK_LO_LEADING );
    Delay_ms( 200 );
    
    UART1_Init( 9600 );
    Delay_ms( 200 );

    OZONE_CS = 1;
}

void display_result()
{
    FloatToStr( ozone, text );
    UART1_Write_Text( text );
    UART1_Write_Text( "\r\n" );
}

void measurement()
{
    uint8_t msb;
    uint8_t lsb;

    OZONE_CS = 0;
    msb = SPI_Rd_Ptr( 0xFF );
    lsb = SPI_Rd_Ptr( 0xFF );
    OZONE_CS = 1;

    raw = ( msb & 0x1F );
    raw <<= 7;
    raw |= ( lsb >> 1 );
}

void calculation()
{
    float Vout;
    float Rs;
    float ratio;
    float lgPPM;

    Vout    = ( float )raw / 4096.0 * Vref;
    Rs = Rr / ( Vref - Vout ) * Vout;
    ratio   = Rs / Rr;
    ozone   = ( ( log10( ratio ) *  -0.8 ) + 0.9 ) * 1000;  // Parts per billion
}

/*
 * Main
 ******************************************************************************/
void main()
{
    system_init();

    while( 1 )
    {
        measurement();
        calculation();
        display_result();
        delay_ms( 500 );
    }
}